/*-----------------------------------------------------------------------

File  : ccl_clauses.h

Author: Stephan Schulz

Contents
 
  Clauses - Infrastructure functions

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Thu Apr 16 19:38:16 MET DST 1998
    New
<2> Mon Jan 11 00:05:24 MET 1999
    Added properties, proof_depth and proof_size

-----------------------------------------------------------------------*/

#ifndef CCL_CLAUSES

#define CCL_CLAUSES

#include <ccl_evaluations.h>
#include <ccl_eqnlist.h>
#include <clb_properties.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum 
{
   CPIgnoreProps    = 0,  /* For masking properties out */
   CPInput          = 1,  /* Input clause */
   CPIsProcessed    = 4,   /* Clause has been processed previously */
   CPIsOriented     = 8,   /* Term and literal comparisons are up to
			      date */
   CPDeleteClause   = 16,  /* Clause should be deleted for some reason */
   CPTypeAxiom      = 32,  /* Clause is TPTPAxiom */
   CPTypeHypothesis = 64,  /* Clause is TPTPHypothesis */
   CPTypeConjecture = 128, /* Clause is TPTPConjecture */
   CPIsWatched      = 256, /* Clause is of special interest */
   CPOpFlag         = 512, /* Temporary marker */
   CPTypeMask       = CPTypeAxiom|CPTypeHypothesis|CPTypeConjecture,
   CPIsSelected     = 1024, /* For analysis of selected clauses only */
   CPIsFinal        = 2048, /* Clause is a final clause, i.e. a clause
			       that might be used by a
			       postprocessor. */
   CPIsProofClause  = 4096  /* Clause is part of a successful
			       proof. */
}ClauseProperties;


typedef struct clause_cell
{
   long                  ident;       /* Hopefully unique ident for
					 all clauses created during
					 proof run */
   SysDate               date;        /* ...at which this clause
					 became a demodulator */
   Eqn_p                 literals;    /* List of literals */
   short                 neg_lit_no;  /* Negative literals */
   short                 pos_lit_no;  /* Positive literals */
   Eval_p                evaluations; /* List of evauations */
   struct clausesetcell* set;         /* Is the clause in a set? */
   ClauseProperties      properties;  /* Anything we want to note at
					 the clause? */
   long                  proof_depth; /* How long is the longest
					 derivation chain from this
					 clause to an axiom? */
   long                  proof_size;  /* How many (generating)
					 inferences were necessary to
					 create this clause? */
   PTree_p               children;    /* Which can be removed if this
			  	         clause changes significantly */
   struct clause_cell*   parent1;     /* Parents need to be notified */
   struct clause_cell*   parent2;     /* if their children are removed! */
   struct clause_cell*   pred;        /* For clause sets = doubly  */
   struct clause_cell*   succ;        /*    linked lists */
}ClauseCell, *Clause_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

extern bool ClausesHaveLocalVariables;

#define ClauseSetProp(clause, prop) SetProp((clause), (prop))
#define ClauseDelProp(clause, prop) DelProp((clause), (prop))

/* Are _all_ properties in prop set in clause? */
#define ClauseQueryProp(clause, prop) QueryProp((clause), (prop))

/* Are any properties in prop set in clause? */
#define ClauseIsAnyPropSet(clause, prop) IsAnyPropSet((clause), (prop))


void ClauseSetTPTPType(Clause_p clause, ClauseProperties type);

#define ClauseQueryTPTPType(clause) \
   ((clause)->properties&CPTypeMask)

#define TPTPTypesCombine(type1, type2)\
   ((type1)==CPTypeAxiom?(type2):\
      ((type2)==CPTypeConjecture?CPTypeConjecture:(type1)))


#define ClauseCellAlloc()    (ClauseCell*)SizeMalloc(sizeof(ClauseCell))
#define ClauseCellFree(junk) SizeFree(junk, sizeof(ClauseCell))

Clause_p EmptyClauseAlloc();
Clause_p ClauseAlloc(Eqn_p literals);
void     ClauseFree(Clause_p junk);

#define  ClauseLiteralNumber(clause)\
         ((clause)->pos_lit_no+(clause)->neg_lit_no)

#define  ClauseIsEmpty(clause) (ClauseLiteralNumber(clause)==0)

#define  ClauseIsGoal(clause) (!((clause)->pos_lit_no))
#define  ClauseIsHorn(clause) ((clause)->pos_lit_no <= 1)
#define  ClauseIsUnit(clause) (ClauseLiteralNumber(clause)==1)
#define  ClauseIsDemodulator(clause)\
            (((clause)->pos_lit_no == 1) && \
	     ((clause)->neg_lit_no == 0))
#define  ClauseIsGround(clause) EqnListIsGround(clause->literals)
#define  ClauseIsTautology(clause) \
         EqnListIsTautologic(clause->literals) 
#define  ClauseIsEquational(clause) \
         EqnListIsEquational(clause->literals) 
#define  ClauseIsPureEquational(clause) \
         EqnListIsPureEquational(clause->literals) 

Clause_p ClauseCopy(Clause_p clause, TB_p bank);

void     ClausePrintList(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintAxiom(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintRule(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintGoal(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintQuery(FILE* out, Clause_p clause, bool fullterms);

void     ClausePrint(FILE* out, Clause_p clause, bool fullterms); 

bool     ClauseStartsMaybe(Scanner_p in);
Clause_p ClauseParse(Scanner_p in, TB_p bank);

void     ClauseMarkMaximalTerms(OCB_p ocb, Clause_p clause);

#define  ClauseOrientLiterals(ocb, clause) \
         EqnListOrient((ocb), (clause)->literals)
#define  ClauseMarkMaximalLiterals(ocb, clause) \
         EqnListMaximalLiterals((ocb), (clause)->literals)

void     ClauseMarkRestrictedTerms(Clause_p clause);
#define  ClauseDeleteTermProperties(clause, props) \
         EqnListDeleteTermProperties((clause)->literals, props)

bool     ClauseParentsAreSubset(Clause_p clause1, Clause_p clause2);
void     ClauseDetachParents(Clause_p clause);
void     ClauseRegisterChild(Clause_p clause, Clause_p child);

void     ClauseAddEvalCell(Clause_p clause, Eval_p evaluation);
void     ClauseRemoveEvaluations(Clause_p clause);

double   ClauseWeight(Clause_p clause, double max_term_multiplier,
		      double max_literal__multiplier, double
		      pos_multiplier, long vweight, long fweight, bool
		      count_eq_encoding);
double   ClauseOrientWeight(Clause_p clause, double
			    unorientable_literal_multiplier,
			    double max_literal_multiplier, double
			    pos_multiplier, long vweight, long
			    fweight, bool count_eq_encoding);

#define  ClauseDepth(clause) EqnListDepth((clause)->literals)

bool     ClauseNotGreaterEqual(OCB_p ocb,
			       Clause_p clause1, Clause_p clause2);

int      ClauseCompareFun(Clause_p clause1, Clause_p clause2);

#define  NormSubstClause(clause, subst, vars)\
         NormSubstEqnListExcept((clause)->literals,\
				NULL, (subst), (vars));

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





